;
; MotoT81_Charger_T44A.asm
;
; Created: 18.01.2017 11:53:54
; Author : ballhause
;


.include "Defines.inc"
.include "Init.inc"

 ;sbi	Status0, _DebugFlag


MainLoop:
 rcall	Current_Regulation
 rcall	Check_SlotStatus
 rcall	Check_BatteryStatus
 
 rcall	Read_ADC

 sbic	Status0, _DebugFlag
 rcall	Show_ADC_Values

 sbic	Status0, _DebugFlag
 rcall	Put_Display

 rjmp	MainLoop


;--------------------------------------------------------------
Check_SlotStatus:
 sbis	Status0, _SlotCheck
 ret

 cbi	Status0, _SlotCheck
 
 lds	Di0, Voltage_LeftSlot
 lds	Di1, Voltage_LeftSlot+1 
 lds	Di2, Voltage_LeftSlot_Old
 lds	Di3, Voltage_LeftSlot_Old+1
 
 sts	Voltage_LeftSlot_Old, Di0
 sts	Voltage_LeftSlot_Old+1, Di1

 ldi	Temp, 0xF0
 ldi	TempH, 0x03
 cp		Di0, Temp
 cpc	Di1, TempH
 brcc	_LeftSlot_Empty

 sbic	Status0, _LeftSlot_Active
 rjmp	_Check_RightSlot
 
 cp		Di2, Temp
 cpc	Di3, TempH
 brcc	_LeftSlot_Reloaded
 rjmp	_Check_RightSlot

_LeftSlot_Reloaded:
 sbi	Status0, _LeftSlot_Active
 clr	LeftSlot_NegativeCT
 sts	Voltage_LeftSlot_Max, Null
 sts	Voltage_LeftSlot_Max+1, Null
 andi	LEDs, 0b00000111
 ori	LEDs, 0b00100000						;Left blue LED
 rjmp	_Check_RightSlot

_LeftSlot_Empty:
 cbi	Status0, _LeftSlot_Active
 andi	LEDs, 0b00000111
 ori	LEDs, 0b00001000						;Left red LED


_Check_RightSlot:
 lds	Di0, Voltage_RightSlot
 lds	Di1, Voltage_RightSlot+1 
 lds	Di2, Voltage_RightSlot_Old
 lds	Di3, Voltage_RightSlot_Old+1
 
 sts	Voltage_RightSlot_Old, Di0
 sts	Voltage_RightSlot_Old+1, Di1

 ldi	Temp, 0xF0
 ldi	TempH, 0x03
 cp		Di0, Temp
 cpc	Di1, TempH
 brcc	_RightSlot_Empty

 sbic	Status0, _RightSlot_Active
 rjmp	_Check_SlotEnd

 cp		Di2, Temp
 cpc	Di3, TempH
 brcc	_RightSlot_Reloaded
 rjmp	_Check_SlotEnd

_RightSlot_Reloaded:
 sbi	Status0, _RightSlot_Active
 clr	RightSlot_NegativeCT
 sts	Voltage_RightSlot_Max, Null
 sts	Voltage_RightSlot_Max+1, Null
 andi	LEDs, 0b00111000
 ori	LEDs, 0b00000100						;Right blue LED
 ret

_RightSlot_Empty:
 cbi	Status0, _RightSlot_Active
 andi	LEDs, 0b00111000
 ori	LEDs, 0b00000001						;Right red LED
 
_Check_SlotEnd:
 ret


;--------------------------------------------------------------
Check_BatteryStatus:
 sbis	Status0, _BatteryStatus
 ret

 cbi	Status0, _BatteryStatus
  
 lds	Temp, Voltage_LeftSlot
 lds	TempH, Voltage_LeftSlot+1
 lds	Temp3, Voltage_LeftSlot_Max
 lds	Temp4, Voltage_LeftSlot_Max+1

 cp		Temp, Temp3
 cpc	TempH, Temp4
 breq	_Left_Status1
 brcs	_Left_Status2
  
_Left_Status0:
 sts	Voltage_LeftSlot_Max, Temp
 sts	Voltage_LeftSlot_Max+1, TempH
 rjmp	_BatteryStatus1

_Left_Status1:
 inc	LeftSlot_NegativeCT
 rjmp	_Left_Status0

_Left_Status2:
 subi	LeftSlot_NegativeCT, -3
 

_BatteryStatus1:
 lds	Temp, Voltage_RightSlot
 lds	TempH, Voltage_RightSlot+1
 lds	Temp3, Voltage_RightSlot_Max
 lds	Temp4, Voltage_RightSlot_Max+1

 cp		Temp, Temp3
 cpc	TempH, Temp4
 breq	_Right_Status1
 brcs	_Right_Status2
 
_Right_Status0:
 sts	Voltage_RightSlot_Max, Temp				
 sts	Voltage_RightSlot_Max+1, TempH
 rjmp	_BatteryStatus2
  
_Right_Status1:
 inc	RightSlot_NegativeCT
 rjmp	_Right_Status0

_Right_Status2:
 subi	RightSlot_NegativeCT, -3

_BatteryStatus2:
 cpi	LeftSlot_NegativeCT, 7
 brcs	_BatteryStatus3
 cbi	Status0, _LeftSlot_Active
 clr	LeftSlot_NegativeCT
 andi	LEDs, 0b00000111
 ori	LEDs, 0b00010000						;Left slot green LED
 sts	Voltage_LeftSlot_Old, Null
 sts	Voltage_LeftSlot_Old+1, Null


_BatteryStatus3:
 cpi	RightSlot_NegativeCT, 7
 brcs	_BatteryStatus4
 cbi	Status0, _RightSlot_Active
 clr	RightSlot_NegativeCT
 andi	LEDs, 0b00111000
 ori	LEDs, 0b00000010						;Right slot green LED
 sts	Voltage_RightSlot_Old, Null
 sts	Voltage_RightSlot_Old+1, Null


_BatteryStatus4:
 ret



 


;--------------------------------------------------------------
Current_Regulation:
 sbis	Status0, _Measure_Complete
 ret

 cbi	Status0, _Measure_Complete

 sbis	Status0, _LeftSlot_Active
 rjmp	_LeftCurrent_Down

 lds	Temp,  Current_LeftSlot 
 lds	TempH,  Current_LeftSlot+1

 ldi	Temp3, high(100)									;left Slot current in mA
 cpi	Temp, low(100)										
 cpc	TempH, Temp3

 breq	_LeftCurrent_OK
 brcc	_LeftCurrent_Down

_LeftCurrent_Up:
 cpi	LeftSlot_PWM, 120
 brcs	_LeftCurrent_Store
 dec	LeftSlot_PWM
 rjmp	_LeftCurrent_Store

_LeftCurrent_Down:
 cpi	LeftSlot_PWM, 254
 breq	_LeftCurrent_Store
 inc	LeftSlot_PWM


_LeftCurrent_Store:
 out	OCR0B, LeftSlot_PWM


_LeftCurrent_OK:
 sbis	Status0, _RightSlot_Active
 rjmp	_RightCurrent_Down
 
 lds	Temp,  Current_RightSlot 
 lds	TempH,  Current_RightSlot+1

 ldi	Temp3, high(100)									;right Slot current in mA
 cpi	Temp, low(100)										
 cpc	TempH, Temp3

 breq	_RightCurrent_OK
 brcc	_RightCurrent_Down

_RightCurrent_Up:
 cpi	RightSlot_PWM, 120
 brcs	_RightCurrent_Store
 dec	RightSlot_PWM
 rjmp	_RightCurrent_Store

_RightCurrent_Down:
 cpi	RightSlot_PWM, 254
 breq	_RightCurrent_Store
 inc	RightSlot_PWM
 
_RightCurrent_Store:
 out	OCR0A, RightSlot_PWM

_RightCurrent_OK:
 ret


 

;--------------------------------------------------------------
Read_ADC:
 sbis	ADCSRA, ADIF
 ret
 
 ldi	Temp, (1<<ADEN)|(0<<ADSC)|(1<<ADIF)|(0<<ADIE)|(1<<ADPS2)|(1<<ADPS0)			;clear interrupt flag
 out	ADCSRA, Temp

 in		Temp, ADMUX
 andi	Temp, 0x03
 cpi	Temp, 0
 breq	_Read_Voltage_LeftSlot
 cpi	Temp, 1
 breq	_Read_Current_LeftSlot
 cpi	Temp, 2
 breq	_Read_Voltage_RightSlot
 cpi	Temp, 3
 breq	_Read_Current_RightSlot
 ret

_Read_Voltage_LeftSlot:
 in		Temp, ADCL
 in		TempH, ADCH
 sts	Voltage_LeftSlot, Temp
 sts	Voltage_LeftSlot+1, TempH
 ldi	Temp, (1<<REFS1)|(0<<MUX1)|(1<<MUX0)															;VCC as reference, ADC1, Left current
 out	ADMUX, Temp
 ret 

_Read_Current_LeftSlot:
 in		Temp, ADCL
 in		TempH, ADCH
 sts	Current_LeftSlot, Temp
 sts	Current_LeftSlot+1, TempH
 ldi	Temp, (0<<REFS1)|(1<<MUX1)|(0<<MUX0)															;VCC as reference, ADC1, Left current
 out	ADMUX, Temp
 ret 
 
_Read_Voltage_RightSlot:
 in		Temp, ADCL
 in		TempH, ADCH
 sts	Voltage_RightSlot, Temp
 sts	Voltage_RightSlot+1, TempH
 ldi	Temp, (1<<REFS1)|(1<<MUX1)|(1<<MUX0)															;VCC as reference, ADC1, Left current
 out	ADMUX, Temp
 ret 

_Read_Current_RightSlot:
 in		Temp, ADCL
 in		TempH, ADCH
 sts	Current_RightSlot, Temp
 sts	Current_RightSlot+1, TempH
 ldi	Temp, (0<<REFS1)|(0<<MUX1)|(0<<MUX0)															;VCC as reference, ADC1, Left current
 out	ADMUX, Temp
 
 sbi	Status0, _Measure_Complete
 ret 




.include "ISR.inc"
.include "Debug.inc"

//End of File//